<?php
/**
 * Plugin Name: EventPrime - Event Calendar Management
 * Plugin URI: http://eventprime.net
 * Description: EventPrime is a complete event management system featuring a backend calendar for event creation, event sites, performers, and many more.
 * Version: 1.2.4
 * Author: EventPrime
 * Text Domain: eventprime-event-calendar-management
 * Domain Path: /languages
 * Author URI: http://eventprime.net
 * Requires at least: 4.8
 * Tested up to: 5.2
 *
 */
if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

if (!class_exists('Event_Magic')) {

    class Event_Magic {

        public $version = '1.2.4';
        public $request_type;
        public $errors = array();
        public $extensions = array();
        protected static $instance = null;

        /**
         *
         * Ensures only one instance of Event_Magic is loaded or can be loaded.
         *
         * @static
         * @return Event_Magic - Main instance.
         */
        public static function instance() {
            if (is_null(self::$instance)) {
                self::$instance = new self();
                self::$instance->define_constants();
                self::$instance->load_textdomain();
                self::$instance->includes();
                self::$instance->init_hooks();
                add_action('plugins_loaded', array(self::$instance, 'plugin_loaded'));
            }
            return self::$instance;
        }

        private function define_constants() {
            $this->define('EM_VENUE_TYPE_TAX', 'em_venue');
            $this->define('EM_EVENT_POST_TYPE', 'em_event');
            $this->define('EM_PERFORMER_POST_TYPE', 'em_performer');
            $this->define('EM_BOOKING_POST_TYPE', 'em_booking');
            $this->define('EM_EVENT_TYPE_TAX', 'em_event_type');
            $this->define('EM_EVENT_VENUE_TAX', 'em_venue');
            $this->define('EM_GLOBAL_SETTINGS', 'em_global_settings');
            $this->define('EM_DB_VERSION', 'emagic_db_version');
            $this->define('EM_PAGINATION_LIMIT', 8);
            $this->define('EM_DEFAULT_CURRENCY', 'USD');
            $this->define('EM_BASE_URL', plugin_dir_url(__FILE__));
            $this->define('EM_BASE_FRONT_IMG_URL', plugin_dir_url(__FILE__) . 'includes/templates/images/');
            $this->define('EM_BASE_DIR', plugin_dir_path(__FILE__));
            $this->define('EM_REQ_EXT_MCRYPT', 1);
            $this->define('EM_REQ_EXT_CURL', 2);
        }

        public function define($name, $value) {
            if (!defined($name)) {
                define($name, $value);
            }
        }

        public function includes() {
            include_once('includes/class-em-factory.php');
            include_once('includes/class-em-constants.php');
            include_once('includes/class-em-raw-request.php');
            include_once('includes/class-do-action.php');
            include_once('includes/em-core-functions.php');
            include_once('includes/models/class-em-model.php');
            include_once( 'includes/class-em-install.php' );

            // Dao
            include_once('includes/dao/class-em-term.php');
            include_once('includes/dao/class-em-post.php');
            include_once('includes/dao/class-em-venue.php');
            include_once('includes/dao/class-em-performer.php');
            include_once('includes/dao/class-em-event-type.php');
            include_once('includes/dao/class-em-event.php');
            include_once('includes/dao/class-em-event.php');
            include_once('includes/dao/class-em-global-settings.php');
            include_once('includes/dao/class-em-booking.php');

            if (is_admin()){
                include_once( 'includes/admin/class-em-admin.php' );
            } 
            else
            {
                include_once('includes/class-em-public.php');
            }

            // Models
            include_once('includes/models/class-em-base-model.php');
            include_once('includes/models/class-em-array-model.php');
            include_once('includes/models/class-em-venue.php');
            include_once('includes/models/class-em-event.php');
            include_once('includes/models/class-em-performer.php');
            include_once('includes/models/class-em-event-type.php');
            include_once('includes/models/class-em-global-settings.php');
            include_once('includes/models/class-em-booking.php');
            include_once( 'includes/class-em-post-types.php' );
            include_once( 'includes/class-em-shortcodes.php' );

            // Services
            include_once('includes/service/class-em-payment.php');
            include_once('includes/service/class-em-event.php');
            include_once('includes/service/class-em-performer.php');
            include_once('includes/service/class-em-venue.php');
            include_once('includes/service/class-em-event-type.php');
            include_once('includes/service/class-em-user.php');
            include_once('includes/service/class-em-booking.php');
            include_once('includes/service/class-em-booking.php');
            include_once('includes/service/class-em-notification.php');
            include_once('includes/service/class-em-print.php');
            include_once('includes/service/class-em-setting.php');
            include_once('includes/service/class-em-extensions.php');
            include_once('includes/service/class-em-paypal.php');

            // Widgets
            include_once('includes/widgets/event_calendar.php');
            include_once('includes/widgets/venue_map.php');
            include_once('includes/widgets/event_slider.php');
            include_once('includes/widgets/event_countdown.php');
            
            include_once( 'includes/class-em-ajax.php' );
        }

        private function init_hooks() {
            add_action('init', 'em_check_event_status'); // Automatically expiring events that are passed.
            add_action('init', 'em_delete_tmp_bookings'); // Deleting temporary bookings.
            add_action('init', 'em_redirect_event_posts');
            add_action('admin_notices', 'em_check_required_pages');
            register_activation_hook(__FILE__, array('EventM_Install', 'install'));
        }
        
        public function load_textdomain(){
            load_plugin_textdomain('eventprime-event-calendar-management', false, dirname(plugin_basename(__FILE__)) . '/languages/');
        }
        
        public function plugin_loaded(){
            do_action('event_magic_loaded');
        }
    }

}

/**
 * Main instance of Event_Magic.
 *
 * @return Event_Magic
 */
function event_magic_instance() {
    return Event_Magic::instance();
}

event_magic_instance();
