<?php

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

class EventM_Admin {
    
    private static $instance = null;
    private $current_page= null;
    public $pages = array('event_magic', 'em_add', 'em_event_types', 'em_venues', 'em_bookings', 
                          'em_booking_add', 'em_performers','em_analytics', 'em_global_settings',
                          'em_frontend', 'em_extensions', 'em_event_type_add', 'em_venue_add',
                          'em_dashboard', 'em_sponsers', 'em_social','em_new_event','em_ticket_templates','em_analytics');
    
    public static function get_instance()
    {   
        if (self::$instance === null) {
            self::$instance = new self();
        }
        return self::$instance;
    }
    
    private function __construct() {
        add_action('admin_menu', array($this, 'menus'));
        add_action('admin_enqueue_scripts', array($this, 'enqueue'));
        add_action('admin_init', array($this,'plugin_redirect'));
        $this->current_page= isset($_GET['page']) ? $_GET['page'] : '';
    }

    public function menus() {
        $global_options = get_option(EM_GLOBAL_SETTINGS);
        add_menu_page(__('EventPrime', 'eventprime-event-calendar-management'), __('EventPrime', 'eventprime-event-calendar-management'), 'manage_options', "event_magic", array($this, 'events'), 'dashicons-tickets-alt', '25');
        add_submenu_page("event_magic", __('All Events', 'eventprime-event-calendar-management'), __('All Events', 'eventprime-event-calendar-management'), "manage_options", "event_magic", array($this, 'events'));
        add_submenu_page("", __('Event Dashboard', 'eventprime-event-calendar-management'), __('Event Dashboard', 'eventprime-event-calendar-management'), "manage_options", "em_dashboard", array($this, 'events'));
        add_submenu_page("", __('Performers', 'eventprime-event-calendar-management'), __('Performers', 'eventprime-event-calendar-management'), "manage_options", "em_dashboard", array($this, 'events'));
        add_submenu_page("event_magic", __('Publish Shortcodes', 'eventprime-event-calendar-management'), __('Publish Shortcodes', 'eventprime-event-calendar-management'), "manage_options", "em_frontend", array($this, 'frontend'));
        add_submenu_page("event_magic", __('Attendees', 'eventprime-event-calendar-management'), __('Attendees', 'eventprime-event-calendar-management'), "manage_options", "em_bookings", array($this, 'bookings'));
        add_submenu_page("", __('Attendee', 'eventprime-event-calendar-management'), __('Attendee', 'eventprime-event-calendar-management'), "manage_options", "em_booking_add", array($this, 'bookings'));
        add_submenu_page("event_magic", __('Global Settings', 'eventprime-event-calendar-management'), __('Global Settings', 'eventprime-event-calendar-management'), "manage_options", "em_global_settings", array($this, 'global_settings'));
        add_submenu_page("", __('Performers', 'eventprime-event-calendar-management'), __('Performers', 'eventprime-event-calendar-management'), "manage_options", "em_performers", array($this, 'performers'));
        add_submenu_page("", __('Event Types', 'eventprime-event-calendar-management'), __('Event Types', 'eventprime-event-calendar-management'), "manage_options", "em_event_types", array($this, 'event_types'));
        add_submenu_page("", __('New Event Type', 'eventprime-event-calendar-management'), __('New Event Type', 'eventprime-event-calendar-management'), "manage_options", "em_event_type_add", array($this, 'event_types'));
        add_submenu_page("", __('Event Site/Locations', 'eventprime-event-calendar-management'), __('Event Site/Locations', 'eventprime-event-calendar-management'), "manage_options", "em_venues", array($this, 'venues'));
        add_submenu_page("", __('Event Site/Locations', 'eventprime-event-calendar-management'), __('Event Site/Locations', 'eventprime-event-calendar-management'), "manage_options", "em_venue_add", array($this, 'venues'));
        add_submenu_page("", __('Add New Event', 'eventprime-event-calendar-management'), __('Add New Event', 'eventprime-event-calendar-management'), "manage_options", "em_new_event", array($this, 'events'));
        $extensions= event_magic_instance()->extensions; 
        if(!in_array('analytics',$extensions)){
            add_submenu_page("event_magic", __('Analytics', 'eventprime-event-calendar-management'), __('Analytics', 'eventprime-event-calendar-management'), "manage_options", "em_analytics", array($this, 'analytics_banner'));
        }
        if(!in_array('seating',$extensions)){
            add_submenu_page("event_magic", __('Tickets', 'eventprime-event-calendar-management'), __('Tickets', 'eventprime-event-calendar-management'), "manage_options", "em_ticket_templates", array($this, 'ticket_templates_banner'));
        }
        do_action('event_magic_menus');
        add_submenu_page("event_magic", __('Extensions', 'eventprime-event-calendar-management'), __('Extensions', 'eventprime-event-calendar-management'), "manage_options", "em_extensions", array($this, 'extensions'));
    }


    public function enqueue() {
        $this->pages = apply_filters('event_magic_admin_pages', $this->pages);
        if (is_admin() && isset($_REQUEST['page']) && in_array($_REQUEST['page'], $this->pages)) {
            wp_enqueue_script('jquery-ui-datepicker');
            wp_enqueue_script('jquery-ui-sortable');

            wp_register_script('em-angular', EM_BASE_URL . 'includes/js/angular.js');
            wp_register_script('dir-pagination', EM_BASE_URL . 'includes/admin/template/js/dirPagination.js', array('em-angular'));
            wp_register_script('em-angular-module', EM_BASE_URL . 'includes/admin/template/js/em-module.js', array('dir-pagination'));
            wp_localize_script('em-angular-module', 'em_ajax_object', array('ajax_url' => admin_url('admin-ajax.php')));
            wp_register_script('em-google-map', EM_BASE_URL . 'includes/js/em-map.js', array('jquery'));
            wp_register_script('em-timepicker', EM_BASE_URL . 'includes/admin/template/js/timepicker-addon.js', false);
            wp_register_script('em-event-controller', EM_BASE_URL . 'includes/admin/template/js/em-event-controller.js', array('em-angular-module', 'em-timepicker', 'em-google-map'));
            wp_register_script('em-venue-controller', EM_BASE_URL . 'includes/admin/template/js/em-venue-controller.js', array('em-angular-module', 'em-timepicker', 'em-google-map'));
            wp_register_script('em-event-type-controller', EM_BASE_URL . 'includes/admin/template/js/em-event-type-controller.js', array('em-angular-module'));
            wp_register_script('em-booking-controller', EM_BASE_URL . 'includes/admin/template/js/em-booking-controller.js', array('em-angular-module'));
            wp_register_script('em-performer-controller', EM_BASE_URL . 'includes/admin/template/js/em-performer-controller.js', array('em-angular-module'));
            wp_register_script('em-global-settings-controller', EM_BASE_URL . 'includes/admin/template/js/em-global-settings-controller.js', array('em-angular-module'));
            wp_register_script('google_charts', "https://www.gstatic.com/charts/loader.js");
            wp_register_script('moment', EM_BASE_URL . 'includes/templates/js/moment.min.js');
            wp_register_script('em-full-calendar',EM_BASE_URL.'includes/templates/js/calendar-3.9.0.js', array('jquery', 'moment'));
            
            wp_enqueue_script('em-utility', EM_BASE_URL . 'includes/js/em-utility.js', array('jquery'));
            wp_enqueue_script('em-admin-utility', EM_BASE_URL . 'includes/admin/template/js/em-admin.js', array('jquery'));
            wp_localize_script('em-admin-utility','admin_vars',array('admin_url'=>admin_url()));
            wp_enqueue_script('em_joyride_js', plugin_dir_url(__DIR__) . 'admin/template/js/jquery.joyride-2.1.js', false);
            wp_enqueue_style('em_joyride_css', plugin_dir_url(__DIR__) . 'admin/template/css/joyride-2.1.css', false);
            wp_enqueue_style('jquery-style', 'https://ajax.googleapis.com/ajax/libs/jqueryui/1.10.4/themes/smoothness/jquery-ui.css');
            wp_enqueue_script('jquery-ui-draggable', false, array('jquery'), false, false);
            wp_enqueue_script('em-admin-jscolor', plugin_dir_url(__DIR__) . '/admin/template/js/em-jscolor.js', false);
            wp_enqueue_style('em_admin_css', plugin_dir_url(__DIR__) . 'admin/template/css/em_admin.css', false);
            wp_register_style('em-full-calendar-css', EM_BASE_URL . 'includes/templates/css/calendar.min.css');
            wp_enqueue_script('em_font-awesome', EM_BASE_URL . 'includes/js/font_awesome.js');

            do_action('event_magic_admin_enqueues');
        }
    }

    public function events() {
        include( 'class-em-event.php' );
    }

    public function venues() {
        if ('em_venues'==$this->current_page) {
            wp_enqueue_script('em-venue-controller');
            include_once('template/venues.php');
        }
        else if('em_venue_add'=== $this->current_page){
            $term_id= isset($_REQUEST['term_id']) ? absint($_REQUEST['term_id']) : 0; 
            wp_enqueue_media();
            wp_enqueue_script('em-venue-controller'); 
            wp_enqueue_style('jquery-style', 'https://ajax.googleapis.com/ajax/libs/jqueryui/1.10.4/themes/smoothness/jquery-ui.css');
            $gmap_api_key= em_global_settings('gmap_api_key');
            if($gmap_api_key)
              wp_enqueue_script ('google_map_key', 'https://maps.googleapis.com/maps/api/js?key='.$gmap_api_key.'&libraries=places');
            include_once('template/venue_add.php');
        }
    }

    public function bookings() {
        if ('em_bookings'== $this->current_page) {
            wp_enqueue_script('em-booking-controller');
            wp_enqueue_style('jquery-style', 'https://ajax.googleapis.com/ajax/libs/jqueryui/1.10.4/themes/smoothness/jquery-ui.css');
            $tab = isset($_GET['tab']) ? $_GET['tab'] : '';
            if($tab=='view'){
                include_once('template/booking.php');
            }
            else
            {
               include_once('template/bookings.php');
            }
        }
        elseif('em_booking_add'==$this->current_page){
            $post_id= isset($_REQUEST['post_id']) ? absint($_REQUEST['post_id']) : 0; 
            if(empty($post_id))
                return;
            wp_enqueue_script('em-booking-controller');
            include_once('template/booking.php');
        }
    }

    public function event_types() {
        if ('em_event_types'==$this->current_page) {
            wp_enqueue_script('em-event-type-controller');
            include_once('template/event_types.php');
        }
        else if('em_event_type_add'==$this->current_page){
            wp_enqueue_media();
            wp_enqueue_script('jquery-ui-slider');		
            wp_enqueue_style('jquery-ui-css','https://code.jquery.com/ui/1.12.1/themes/base/jquery-ui.css');
            wp_enqueue_script('em-event-type-controller');
            include_once('template/event_type_add.php');
        }
    }

    public function performers() {
        if ('em_performers'==$this->current_page) {
            wp_enqueue_script('em-performer-controller');
            $tab = isset($_GET['tab']) ? $_GET['tab'] : '';
            if($tab=='add'){
                include_once('template/performer_add.php');
            }
            else
            {
               include_once('template/performers.php');
            }
        }
    }

    public function global_settings() {
        if($this->current_page=='em_global_settings'){
            wp_enqueue_script( 'em-global-settings-controller',plugin_dir_url(__DIR__) . '/admin/template/js/em-global-settings-controller.js',false );
            include_once('template/global_settings.php');
        }
    }

    public function frontend() {
        include_once( 'template/frontend.php' );
    }

    public function extensions() {
        include_once( 'template/extensions.php' );
    }
    
    public function plugin_redirect(){
        if (get_option('event_magic_do_activation_redirect', false)) {
            delete_option('event_magic_do_activation_redirect');
            wp_redirect(admin_url("admin.php?page=event_magic"));
            exit;
        }
    }
    
    public function analytics_banner(){
        include_once('template/analytics.php' );
    }
    
    public function ticket_templates_banner(){
        include_once('template/ticket_templates.php' );
    }

}

EventM_Admin::get_instance();
