<?php

if (!defined( 'ABSPATH')){
    exit;
}

class EventM_Notification_Service {
    
    public static function booking_confirmed($booking)
    {  
        $setting_service = EventM_Factory::get_service('EventM_Setting_Service');
        $gs= $setting_service->load_model_from_db();
        
        if(empty($gs->send_booking_confirm_email))
           return;
        
        $booking_service = EventM_Factory::get_service('EventM_Booking_Service');
        if(!$booking instanceof EventM_Booking_Model){
            $booking= $booking_service->load_model_from_db($booking);
        }
        if(empty($booking->id))
            return;
        $event_service = EventM_Factory::get_service('EventM_Service');
        $event= $event_service->load_model_from_db($booking->event);
        $venue_service= EventM_Factory::get_service('EventM_Venue_Service');
        $venue= $venue_service->load_model_from_db($event->venue);
        if(empty($venue->id)){
            $venue= false;
        }
        
        if(empty($event->id))
            return;
        
        self::configure_mail();
        $user= get_user_by('ID',$booking->user);
        $mail_body =  $gs->booking_confirmed_email;
        $mail_body= isset($booking->order_info['seat_sequences']) ? str_replace("(Seat No.)",implode(',',$booking->order_info['seat_sequences']), $mail_body) : str_replace("(Seat No.)", "Standing Event", $mail_body);
        $mail_body = str_replace("#ID",$booking->id, $mail_body);
        $mail_body = str_replace("Event Name", $event->name, $mail_body);
        $mail_body = str_replace("Venue Name",empty($venue->id) ? '' : $venue->name, $mail_body);
        $mail_body = str_replace("Event Venue",empty($venue->id) ? '' : $venue->address, $mail_body);
        $mail_body = str_replace("$(Subtotal)",($booking->order_info['quantity']*$booking->order_info['item_price'])-$booking->order_info['discount'], $mail_body);
        $mail_body = str_replace("(Quantity)", $booking->order_info['quantity'], $mail_body);
        $mail_body = str_replace("$(Price)", $booking->order_info['item_price'], $mail_body);
        $mail_body = str_replace("$(Discount)", $booking->order_info['discount'], $mail_body);
        $mail_body = str_replace("(User Email)", $user->email, $mail_body);
       
        /* send Mail to User */    
        $to = $user->user_email;
        $subject = __('Booking Confirmation','eventprime-event-calendar-management');
        wp_mail( $to, $subject, $mail_body);

        /* send Mail to Admin */   
        $mail_body = file_get_contents(EM_BASE_URL.'includes/mail/admin_confirm.html');
        $to = get_option('admin_email'); 
        $subject = __('Booking Confirmation','eventprime-event-calendar-management');
        $mail_body = str_replace("#ID", $booking->id, $mail_body);
        $mail_body = str_replace("(User Email)", $user->email, $mail_body);
        wp_mail( $to, $subject, $mail_body);
    }
    
    public static function booking_pending($booking)
    {
        $setting_service = EventM_Factory::get_service('EventM_Setting_Service');
        $gs= $setting_service->load_model_from_db();
        
        $booking_service = EventM_Factory::get_service('EventM_Booking_Service');
        if(!$booking instanceof EventM_Booking_Model){
            $booking= $booking_service->load_model_from_db($booking);
        }
        if(empty($booking->id))
            return;
        
        $event_service = EventM_Factory::get_service('EventM_Service');
        $event= $event_service->load_model_from_db($booking->event);
        $venue_service= EventM_Factory::get_service('EventM_Venue_Service');
        $venue= $venue_service->load_model_from_db($event->venue);
        if(empty($venue->id)){
            $venue= false;
        }
        
        self::configure_mail();
        $user= get_user_by('ID', $booking->user);
        if(empty($user))
            return false;
        
        $mail_body =  $gs->booking_pending_email;
        $mail_body= isset($booking->order_info['seat_sequences']) ? str_replace("(Seat No.)",implode(',',$booking->order_info['seat_sequences']), $mail_body) : str_replace("(Seat No.)", "Standing Event", $mail_body);
        $mail_body = str_replace("ID",$booking->id, $mail_body);
        $mail_body = str_replace("Event Name", $event->name, $mail_body);
        $mail_body = str_replace("Venue Name",empty($venue->id) ? '' : $venue->name, $mail_body);
        $mail_body = str_replace("Event Venue",empty($venue->id) ? '' : $venue->address, $mail_body);
        $mail_body = str_replace("$(Subtotal)", ($booking->order_info['quantity']*$booking->order_info['item_price'])-$booking->order_info['discount'], $mail_body);
        $mail_body = str_replace("(Quantity)", $booking->order_info['quantity'], $mail_body);
        $mail_body = str_replace("$(Price)", $booking->order_info['item_price'], $mail_body);
        $mail_body = str_replace("$(Discount)", $booking->order_info['discount'], $mail_body);
        $to = $user->user_email;
        $subject = __('Booking Pending','eventprime-event-calendar-management');
        wp_mail( $to, $subject, $mail_body );
        
        // Admin email
        $to = get_option('admin_email');
        $subject = __('Booking Pending','eventprime-event-calendar-management');        
        $body = sprintf(__('User %s has Booking Pending with Booking ID #%d.','eventprime-event-calendar-management'),$user->user_email,$booking->id);
        wp_mail( $to, $subject, $body);
    }

    
    public static function booking_cancel($booking)
    {   
        $setting_service = EventM_Factory::get_service('EventM_Setting_Service');
        $gs= $setting_service->load_model_from_db();
        if(empty($gs->send_booking_cancellation_email))
           return; 
        $booking_service = EventM_Factory::get_service('EventM_Booking_Service');
        if(!$booking instanceof EventM_Booking_Model){
            $booking= $booking_service->load_model_from_db($booking);
        }
        if(empty($booking->id))
            return;
       
        $event_service = EventM_Factory::get_service('EventM_Service');
        $event= $event_service->load_model_from_db($booking->event);
        
        self::configure_mail();
        $user= get_user_by('ID', $booking->user);
        if(empty($user))
            return false;            
            
        $mail_body =  $gs->booking_cancelation_email;
        $mail_body= isset($booking->order_info['seat_sequences']) ? str_replace("(Seat No.)",implode(',',$booking->order_info['seat_sequences']), $mail_body) : str_replace("(Seat No.)", "Standing Event", $mail_body);
        $mail_body = str_replace("#ID",$booking->id, $mail_body);
        $mail_body = str_replace("Event Name", $event->name, $mail_body);
        $mail_body = str_replace("Event Venue",empty($venue->id) ? '' : $venue->address, $mail_body);
        $mail_body = str_replace("$(Subtotal)", ($booking->order_info['quantity']*$booking->order_info['item_price'])-$booking->order_info['discount'], $mail_body);
        $mail_body = str_replace("(Quantity)", $booking->order_info['quantity'], $mail_body);
        $mail_body = str_replace("$(Price)", $booking->order_info['item_price'], $mail_body);
        $mail_body = str_replace("$(Discount)", $booking->order_info['discount'], $mail_body);
        $to = $user->user_email;
        $subject = __('Booking Cancellation','eventprime-event-calendar-management');
        wp_mail( $to, $subject, $mail_body ); 
        
        // Admin email
        $mail_body = file_get_contents(EM_BASE_URL.'includes/mail/admin_cancellation.html');  
        $mail_body = str_replace("Event Name", $event->name, $mail_body);
        
        $admin_email = get_option('admin_email'); 
        $mail_body = str_replace("#ID",$booking->id, $mail_body);
        $mail_body = str_replace("(User Email)",$user->user_email, $mail_body);
        $to = $admin_email;
        $subject = __('Booking Cancellation','eventprime-event-calendar-management');
        wp_mail( $to, $subject, $mail_body); 
    }
    
    
    public static function reset_password_mail($booking,$new_user_password)
    {   
        $setting_service = EventM_Factory::get_service('EventM_Setting_Service');
        $gs= $setting_service->load_model_from_db();
        $booking_service = EventM_Factory::get_service('EventM_Booking_Service');
        if(!$booking instanceof EventM_Booking_Model){
            $booking= $booking_service->load_model_from_db($booking);
        }
                
        if(empty($booking->id))
            return;
        
        self::configure_mail();
        $user= get_user_by('ID', $booking->user);
        if(empty($user))
            return false;

        $mail_body= $gs->reset_password_mail;
        $mail_body = str_replace("@username",$user->user_email,$mail_body);
        $mail_body = str_replace("@password",$new_user_password,$mail_body);   
         
        
        $to = $user->user_email;
        $subject = __('New Password','eventprime-event-calendar-management');
        $body = $mail_body;
        wp_mail( $to, $subject, $body );
        
        $admin_email = get_option('admin_email'); 
        $to = $admin_email;
        $subject = __('Reset User Password','eventprime-event-calendar-management');        
        $body = sprintf(__('Password of user %s is Reset.','eventprime-event-calendar-management'),$user->user_email);
        wp_mail( $to, $subject, $body);
    }
    
    public static function booking_refund($booking)
    {
        self::configure_mail();
        $booking_service = EventM_Factory::get_service('EventM_Booking_Service');
        if(!$booking instanceof EventM_Booking_Model){
            $booking= $booking_service->load_model_from_db($booking);
        }
        if(empty($booking->id))
            return false;
        $user= get_user_by('ID', $booking->user);
        if(empty($user))
            return false;
        $setting_service = EventM_Factory::get_service('EventM_Setting_Service');
        $gs= $setting_service->load_model_from_db();
        $event_service = EventM_Factory::get_service('EventM_Service');
        $event= $event_service->load_model_from_db($booking->event);
        
        $venue_service= EventM_Factory::get_service('EventM_Venue_Service');
        $venue= $venue_service->load_model_from_db($event->venue);
        if(empty($venue->id)){
            $venue= false;
        }
        
        
        $mail_body =  $gs->booking_refund_email; 
        $mail_body= isset($booking->order_info['seat_sequences']) ? str_replace("(Seat No.)",implode(',',$booking->order_info['seat_sequences']), $mail_body) : str_replace("(Seat No.)", "Standing Event", $mail_body);
        $mail_body = str_replace("#ID",$booking->id, $mail_body);
        $mail_body = str_replace("Event Name", $event->name, $mail_body);
        $mail_body = str_replace("Event Venue",empty($venue->id) ? '' : $venue->address, $mail_body);
        $mail_body = str_replace("Venue Name",empty($venue->id) ? '' : $venue->name, $mail_body);
        $mail_body = str_replace("$(Subtotal)",($booking->order_info['quantity']*$booking->order_info['item_price'])-$booking->order_info['discount'], $mail_body);
        $mail_body = str_replace("(Quantity)", $booking->order_info['quantity'], $mail_body);
        $mail_body = str_replace("$(Price)", $booking->order_info['item_price'], $mail_body);
        $mail_body = str_replace("$(Discount)", $booking->order_info['discount'], $mail_body);
        
        $to = $user->user_email;
        $subject = __('Booking Refund','eventprime-event-calendar-management');
        wp_mail($to, $subject, $mail_body );
        
        //Admin notification
        $admin_email = get_option('admin_email');  
        $to = $admin_email; 
        $subject = sprintf(__('Booking Refund on Booking ID# %d','eventprime-event-calendar-management'),$booking->id);        
        $body = sprintf(__('A refund of %s%0.2f has been issued to booking #%d for %s','eventprime-event-calendar-management'),$booking->order_info['currency'],($booking->order_info['quantity']*$booking->order_info['item_price'])-$booking->order_info['discount'],$booking->id,$event->name);
        wp_mail($to, $subject, $body);
    }
    
     public static function user_registration($user_data=null ) { 
        self::configure_mail();
        $setting_service = EventM_Factory::get_service('EventM_Setting_Service');
        $gs= $setting_service->load_model_from_db();
        $mail_body= $gs->registration_email_content;
        $mail_body = str_replace("@username",$user_data->email,$mail_body);
        $mail_body = str_replace("@password",$user_data->password,$mail_body);
        
        //$body_content .= "Your auto generated password is ".$user_data->password;
        $registration_email_subject= $gs->registration_email_subject;
        
        if(empty($registration_email_subject)){
            $registration_email_subject= get_bloginfo('name');
        }
        
        if(!empty($user_data)){
            wp_mail( $user_data->email, $registration_email_subject, $mail_body );
            $admin_email = get_option('admin_email'); 
            $to = $admin_email;
            $subject = __('New User Registered','eventprime-event-calendar-management'); 
            $body = sprintf(__('New user %s has Registered','eventprime-event-calendar-management'),$user_data->email);
            wp_mail( $to, $subject, $body);
            return true;
        }
        return false;
    }
    
    private static function configure_mail()
    {  
        add_filter('wp_mail_content_type', 'em_set_mail_content_type_html');
        add_filter( 'wp_mail_from', 'em_set_mail_from' );
        add_filter( 'wp_mail_from_name', 'em_set_mail_from_name' );
    }  
}