<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class EventM_Setting_Service {
    
    private $dao;
    private static $instance = null; 
    
    private function __construct() {
        $this->dao= new EventM_Global_Settings_DAO();
    }
    
    public static function get_instance()
    {   
        if (self::$instance === null) {
            self::$instance = new self();
        }
        return self::$instance;
    }
    
    /*
     * Loading settings data on REST call
     */
    public function load_edit_page()
    {
        $options=  $this->load_model_from_db();
        $options= apply_filters('em_load_gs_ext_options',$options);
        $options->pages= $this->load_global_pages();
        $options->base_path= EM_BASE_URL.'includes/admin/template/';
        $options->currencies= em_array_to_options(EventM_Constants::$currencies);
        return $options;
    }

    public function save($model)
    {   
       $model= $this->format_model_to_save($model);
       $this->dao->save($model);
    }
    
    
    public function load_global_pages(){
        $list= array();
        $pages = get_pages(array(
            'post_status'=>'publish',
             'numberposts'=>-1,
            ) );
         
         if (!empty($pages)) {
           foreach ($pages as $page) {
                $tmp = new stdClass();
                $tmp->id = $page->ID;
                $tmp->name = $page->post_title;
                $list[] = $tmp;
            }
       }
       return $list; 
    }
    
    public function load_model_from_db()
    {
        $model= $this->dao->get();
        $model->gcal_sharing= absint($model->gcal_sharing);
        $model->events_page= absint($model->events_page);
        $model->event_types= absint($model->event_types);
        $model->performers_page= absint($model->performers_page);
        $model->profile_page= absint($model->profile_page);
        $model->venues_page= absint($model->venues_page);
        $model->booking_page= absint($model->booking_page);
        $model->paypal_processor= absint($model->paypal_processor);
        $model->payment_test_mode= absint($model->payment_test_mode);
        return $model;
    }
    
    private function format_model_to_save($model){
        $model->gcal_sharing= absint($model->gcal_sharing);
        $model->events_page= absint($model->events_page);
        $model->event_types= absint($model->event_types);
        $model->performers_page= absint($model->performers_page);
        $model->profile_page= absint($model->profile_page);
        $model->venues_page= absint($model->venues_page);
        $model->booking_page= absint($model->booking_page);
        $model->paypal_processor= empty($model->paypal_processor) ? 0 : 1;
        $model->payment_test_mode= empty($model->payment_test_mode) ? 0 : 1;
        $model->paypal_email=  sanitize_email($model->paypal_email);
        $model->paypal_api_username= sanitize_text_field($model->paypal_api_username);
        $model->paypal_api_password= sanitize_text_field($model->paypal_api_password);
        $model->paypal_api_sig= sanitize_text_field($model->paypal_api_sig);
        return $model;
    }
    
    public function map_request_to_model($model=null)
    {  
        $settings= new EventM_Global_Settings_Model();
        $data= (array) $model;
        
        if(!empty($data) && is_array($data))
        {
            foreach($data as $key=>$val)
            {
                if(property_exists($settings, $key)){
                    $settings->{$key}= $val;
                }
            }
        }
        return $settings;
    }
        
}